"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var const_1 = require("./const");
var utils_1 = require("./utils");
var mappedData = [
    {
        color: 'limegreen',
        endAngle: 1.0471975511965976,
        index: 0,
        label: undefined,
        middleAngle: 0.5235987755982988,
        moved: false,
        percentageValue: 0.16666666666666666,
        startAngle: 0,
        value: 10
    },
    {
        color: 'mediumvioletred',
        endAngle: 4.1887902047863905,
        index: 1,
        label: undefined,
        middleAngle: 2.617993877991494,
        moved: false,
        percentageValue: 0.5,
        startAngle: 1.0471975511965976,
        value: 30
    },
    {
        color: 'mediumpurple',
        endAngle: 6.283185307179586,
        index: 2,
        label: undefined,
        middleAngle: 5.235987755982988,
        moved: false,
        percentageValue: 0.3333333333333333,
        startAngle: 4.1887902047863905,
        value: 20
    }
];
describe(utils_1.checkIsDataArrayOfNumbers, function () {
    it('should return true if its array of numbers', function () {
        var result = (0, utils_1.checkIsDataArrayOfNumbers)([1, 10, 50, 40]);
        expect(result).toEqual(true);
    });
    it('should return false if its array of objects', function () {
        var result = (0, utils_1.checkIsDataArrayOfNumbers)([{ value: 1 }, { value: 2 }]);
        expect(result).toEqual(false);
    });
});
describe(utils_1.checkIsUserData, function () {
    it('should return true if its array of UserData', function () {
        var result = (0, utils_1.checkIsUserData)([{ test: 1 }, { test: 2 }]);
        expect(result).toEqual(false);
    });
    it('should return false is not array of UserData', function () {
        var result = (0, utils_1.checkIsUserData)([{ value: 1 }, { value: 2 }]);
        expect(result).toEqual(true);
    });
});
describe(utils_1.createEndWallPath, function () {
    it('Creates end wall svg path', function () {
        var result = (0, utils_1.createEndWallPath)(0, 120, 120, 40, 0.8);
        expect(result).toEqual('M 96 0 L 96 40 L 120 40 L 120 0 z');
    });
});
describe(utils_1.createInnerPath, function () {
    it('Creates inner path', function () {
        var result = (0, utils_1.createInnerPath)(0, const_1.pi / 2, 120, 120, 40, 0.8);
        expect(result).toEqual('M -96 1.175660927181459e-14 A 96 96 0 0 1 -96 1.175660927181459e-14 ' +
            'L -96 40.000000000000014 A 96 96 0 0 0 -96 40.000000000000014 z');
    });
});
describe(utils_1.createLabelPath, function () {
    it('Creates a label path', function () {
        var result = (0, utils_1.createLabelPath)(const_1.pi / 3, 120, 120, 20, 100, 12);
        expect(result).toEqual({
            path: 'M 60.000000000000014 113.92304845413263 L 80.00000000000001 133.92304845413264 l 20 0',
            x: 101.00000000000001,
            y: 137.55941209049627
        });
    });
});
describe(utils_1.createOuterPath, function () {
    it('should create an outer path', function () {
        var result = (0, utils_1.createOuterPath)(0, const_1.pi / 2, 120, 120, 40);
        expect(result)
            .toEqual('M 120 40 A 120 120 0 0 1 7.34788079488412e-15 160 L 7.34788079488412e-15 120 A 120 120 0 0 0 120 0 z');
    });
});
describe(utils_1.createStartWallPath, function () {
    it('should create a start wall path', function () {
        var result = (0, utils_1.createStartWallPath)(0, 120, 120, 40, 0.8);
        expect(result).toEqual('M 96 0 L 96 40 L 120 40 L 120 0 z');
    });
});
describe(utils_1.createTopPath, function () {
    it('should create top svg path', function () {
        var result = (0, utils_1.createTopPath)(0, const_1.pi / 2, 10, 50, 5);
        expect(result).toEqual('M 10 0 A 10 50 0 0 1 6.123233995736766e-16 50 L 3.061616997868383e-15' +
            ' 250 A 50 250 0 00 50 0 z');
    });
});
describe(utils_1.getMiddleAngle, function () {
    it('should return middle of the angel', function () {
        var result = (0, utils_1.getMiddleAngle)(10, 80);
        expect(result).toEqual(45);
    });
});
describe(utils_1.getNewPosition, function () {
    it('should return new position', function () {
        var result = (0, utils_1.getNewPosition)(const_1.pi / 4, 10, 50, 5);
        expect(result).toEqual([35.35533905932738, 176.77669529663686]);
    });
});
describe(utils_1.mapData, function () {
    it('should return mapped data for pie slices', function () {
        var data = [
            { value: 10, label: 'orange', color: 'orange' },
            { value: 30, label: 'lemon', color: 'yellow' },
            { value: 20, label: 'apple', color: 'green' }
        ];
        var result = (0, utils_1.mapData)(data);
        expect(result).toEqual([
            {
                color: 'orange',
                endAngle: 1.0471975511965976,
                index: 0,
                label: 'orange',
                middleAngle: 0.5235987755982988,
                moved: false,
                percentageValue: 0.16666666666666666,
                startAngle: 0,
                value: 10
            },
            {
                color: 'yellow',
                endAngle: 4.1887902047863905,
                index: 1,
                label: 'lemon',
                middleAngle: 2.617993877991494,
                moved: false,
                percentageValue: 0.5,
                startAngle: 1.0471975511965976,
                value: 30
            },
            {
                color: 'green',
                endAngle: 6.283185307179586,
                index: 2,
                label: 'apple',
                middleAngle: 5.235987755982988,
                moved: false,
                percentageValue: 0.3333333333333333,
                startAngle: 4.1887902047863905,
                value: 20
            }
        ]);
    });
    it('should return mapped data for pie slices without all keys', function () {
        var data = [
            { value: 10 },
            { value: 30 },
            { value: 20 }
        ];
        var result = (0, utils_1.mapData)(data);
        expect(result).toEqual(mappedData);
    });
});
describe(utils_1.mapRowData, function () {
    it('should return mapped row data to user data', function () {
        var data = [10, 40, 30, 20];
        var result = (0, utils_1.mapRowData)(data);
        expect(result).toEqual([
            { value: 10 },
            { value: 40 },
            { value: 30 },
            { value: 20 }
        ]);
    });
});
describe(utils_1.moveElement, function () {
    it('should toggle move value of element', function () {
        var result = (0, utils_1.moveElement)(0, [
            {
                color: 'limegreen',
                endAngle: 1.0471975511965976,
                index: 0,
                label: undefined,
                middleAngle: 0.5235987755982988,
                moved: false,
                percentageValue: 0.16666666666666666,
                startAngle: 0,
                value: 10
            },
            {
                color: 'mediumvioletred',
                endAngle: 4.1887902047863905,
                index: 1,
                label: undefined,
                middleAngle: 2.617993877991494,
                moved: false,
                percentageValue: 0.5,
                startAngle: 1.0471975511965976,
                value: 30
            }
        ]);
        expect(result).toEqual([
            {
                color: 'limegreen',
                endAngle: 1.0471975511965976,
                index: 0,
                label: undefined,
                middleAngle: 0.5235987755982988,
                moved: true,
                percentageValue: 0.16666666666666666,
                startAngle: 0,
                value: 10
            },
            {
                color: 'mediumvioletred',
                endAngle: 4.1887902047863905,
                index: 1,
                label: undefined,
                middleAngle: 2.617993877991494,
                moved: false,
                percentageValue: 0.5,
                startAngle: 1.0471975511965976,
                value: 30
            }
        ]);
    });
});
describe(utils_1.createElementPieces, function () {
    it('should create elements for pie', function () {
        var result = (0, utils_1.createElementPieces)(mappedData);
        expect(result).toEqual([
            [
                {
                    color: 'limegreen',
                    endAngle: 1.0471975511965976,
                    index: 0,
                    label: undefined,
                    middleAngle: 0.5235987755982988,
                    moved: false,
                    percentageValue: 0.16666666666666666,
                    startAngle: 0,
                    value: 10
                }
            ],
            [],
            [
                {
                    color: 'mediumvioletred',
                    endAngle: 4.1887902047863905,
                    index: 1,
                    label: undefined,
                    middleAngle: 2.617993877991494,
                    moved: false,
                    percentageValue: 0.5,
                    startAngle: 1.0471975511965976,
                    value: 30
                }
            ],
            [],
            [
                {
                    color: 'mediumpurple',
                    endAngle: 6.283185307179586,
                    index: 2,
                    label: undefined,
                    middleAngle: 5.235987755982988,
                    moved: false,
                    percentageValue: 0.3333333333333333,
                    startAngle: 4.1887902047863905,
                    value: 20
                }
            ]
        ]);
    });
});
